//----------------------------------------------------------------------------//

/* 

Blue Sky:

1. Implement negative assertions.  For each "assert" or "hasAssertion"
method, create the equivalent "deny" or "hasDenial" method.

2. Figure out how to make this stuff useful for composite data sources that
are the databases for XUL templates.

3. Figure out how to make this stuff useful for in-memory data sources.

4. Make this a real implementation of nsIRDFDataSource so that, f.e.,
   adding a data source to a database doesn't require adding the source's
   internal "ds" reference to the underlying XPCOM data source.
*/

//----------------------------------------------------------------------------//
// Global Object Instantiation

// The RDF Service (nsIRDFService)
var gRDFService = 
  Components
    .classes["@mozilla.org/rdf/rdf-service;1"]
      .getService(Components.interfaces.nsIRDFService);
if (!gRDFService) 
  throw("The RDF Service could not be obtained.");

// The RDF Container Utilities Service (nsIRDFContainerUtils)
var gRDFContainerService = 
  Components
    .classes["@mozilla.org/rdf/container-utils;1"]
      .getService(Components.interfaces.nsIRDFContainerUtils);
if (!gRDFContainerService) 
  throw("The RDF Container Utilities Service could not be obtained.");

//----------------------------------------------------------------------------//
// Data Source Object Constructor and Class Properties/Methods

function RDFDataSource(uri, defaultNameSpace, inMemory)
{
  if (uri) {
    // If the data source was already defined, return the existing one;
    // otherwise add the newly created data source to the list.
    if ( typeof RDFDataSource.all[uri] != "undefined" ) 
      return RDFDataSource.all[uri];
    
    this.uri = uri;
    RDFDataSource.all[uri] = this;
  }

  // Set the default name space, either to the one specified by the calling
  // code or to the RDF name space defined in the RDF specification.
  this.defaultNameSpace = 
    defaultNameSpace || "http://www.w3.org/1999/02/22-rdf-syntax-ns#";

  // Create a list of namespaces that may be used in this data source,
  // indexed by their prefixes.  Once added to this list via setNameSpace(),
  // prefixes can be used in place of namespaces to make code easier
  // to write and read.
  this.nameSpaces = new Object();
  
  this.inMemory = inMemory || false;
  if (this.inMemory)
  {
    this.ds =
      Components
        .classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
          .createInstance(Components.interfaces.nsIRDFDataSource);
  }

  // Return the newly created data source.
  return this;
}

// A collection of all data sources, indexed by URI.
RDFDataSource.all = new Object();

RDFDataSource.get = function(uri)
{
  if (typeof this.all[uri] == "undefined")
    return null;
  
  return this.all[uri];
}

RDFDataSource.nodeValue = function(node)
{
  if (!node) return null;
  try 
  {
    node = node.QueryInterface(Components.interfaces.nsIRDFLiteral);
  } 
  catch (e) 
  {
    try 
    {
      node = node.QueryInterface(Components.interfaces.nsIRDFResource);
    }
    catch (e)
    {
      return null;
    }
  }

  return node.Value;
}

RDFDataSource.nodeType = function(node)
{
  // Returns the type (resource or literal) of an RDF node.
  
  if (!node) return null;
  
  var nodeType;
  
  try 
  { 
    node = node.QueryInterface(Components.interfaces.nsIRDFLiteral);
    nodeType = "literal";
  } 
  catch (e) 
  {
    try 
    {
      node = node.QueryInterface(Components.interfaces.nsIRDFResource);
      nodeType = "resource";
    }
    catch (e)
    {
      return null;
    }
  }

  return nodeType;
}

RDFDataSource.generateRandomString = function(count)
{
  // Generates a random sequence of characters.
  
  if (!count) count = 6;
  
  var characters = 
    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789".split("");
  
  var string = "";
  while (string.length < count) 
    string += characters[ Math.floor( Math.random() * characters.length ) ];
  
  return string;
}


//----------------------------------------------------------------------------//
// Data source Manipulation

RDFDataSource.prototype.load = function(observer)
{
  if (typeof this.ds == "undefined") 
    this.ds = gRDFService.GetDataSource(this.uri);

  var remote = this.ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
  if (!remote) return null;

  if (typeof observer != "undefined")
  {
    var sink;
    if (!remote.loaded)
    {
      // The data source is not yet loaded, so get its sink and attach the
      // observer to it so the observer gets notified when the data source
      // load status changes.
      sink = this.ds.QueryInterface(Components.interfaces.nsIRDFXMLSink);
      if (!sink) return null;
      // Note!  The order in which these observers are added is significant.
      // The last observer to be added is the first one to be called, 
      // and we want the data source's own observer to be called first
      // so it can record the time the data source was loaded
      // before other observers want to access that value.
      sink.addXMLSinkObserver(observer);
      sink.addXMLSinkObserver(RDFDataSource.loadObserver);
    }
    else
    {
      // The data source is already loaded, but there must have been some reason
      // this method was called, so act as if the data source had just finished
      // loading and call its observer's onEndLoad method.
      sink = this.ds.QueryInterface(Components.interfaces.nsIRDFXMLSink);
      if (!sink) 
        throw ("RDFDataSource.load: could not access the nsIRDFXMLSink " + 
              " interface for '" + this.uri + "'.");
      observer.onEndLoad(sink);
    }
  }
  
  return true;
}

// Returns true, false, or null depending on whether the data source is loaded,
// not yet loaded, or the load state cannot be determined (either because
// a load attempt has not yet been made via the load() method or because the
// data source cannot be loaded because it is in-memory).
RDFDataSource.prototype.loaded = function()
{
  if (typeof this.ds == "undefined") return null;
  var remote = this.ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
  if (!remote) return null;
  else return remote.loaded;
}

// Observes data sources as they load.  This is a library-specific observer
// for things the library wants done at various stages in the load process.
// Code that wants anything special done should either redefine this method
// or pass a custom observer into the load() method.
RDFDataSource.loadObserver = {
  onBeginLoad: function(aSink) {} , 
  onInterrupt: function(aSink) {} , 
  onResume: function(aSink) {} , 
  onEndLoad: function(aSink)
  { 
    var rawDS = aSink.QueryInterface(Components.interfaces.nsIRDFDataSource);
    var ds = RDFDataSource.all[rawDS.URI];
    
    // Stamp this data source with the date/time it loaded.
    if (ds) ds.loadDate = new Date();
  } , 
  onError: function(aSink, aStatus, aErrorMsg) {}
};

RDFDataSource.prototype.flush = function()
{
  var rds = this.ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
  if (!rds) return false;
  
  rds.Flush();

  return true;
}

RDFDataSource.prototype.refresh = function(url, isSynchronous)
{
  // ??? why does this method take a "URL" parameter?

  if (!isSynchronous) isSynchronous = false;

  if (typeof this.ds == "undefined") 
    this.load();
  else
  {
    var remote = this.ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    if (!remote) return;
    remote.Refresh(isSynchronous);
  }
}

RDFDataSource.prototype.cancelLoad = function()
{
  if (!this.ds) return;
  var sink = this.ds.QueryInterface(Components.interfaces.nsIRDFXMLSink);
  if (!sink) return;
  sink.interrupt();
}

// !!! Remove this function!
RDFDataSource.prototype.addToDatabase = function(aDatabase)
{
  aDatabase.AddDataSource(this.ds);
}

//----------------------------------------------------------------------------//
// Name Space Methods and Utilities

RDFDataSource.prototype.getNameSpace = function(prefix)
{
  var namespace;
  
  if (typeof this.nameSpaces[prefix] != "undefined") 
    namespace = this.nameSpaces[prefix];
    
  return namespace;
}

RDFDataSource.prototype.setNameSpace = function(prefix, uri)
{
  this.nameSpaces[prefix] = uri;
  return true;
}

RDFDataSource.prototype.tagToURI = function(tag)
{
  // Convert tags in the format "NameSpacePrefix:TagName" into fully-qualified 
  // URIs by converting the prefix into its corresponding URI.  F.e. RDF:about
  // converts to "http://www.w3.org/1999/02/22-rdf-syntax-ns#about" when "RDF"
  // is used as a prefix for that RDF name space.  This function only converts
  // tags whose name space prefixes are registered using setNameSpace().

  // Mozilla uses URIs starting with "rdf:#$" to identify containers, and this
  // library also generates random URIs in that format (see generateUniqueURI).
  // These URIs are not tags, but because "rdf" is commonly used as 
  // the namespace prefix for RDF itself, we check for them here and return
  // them untouched so they don't get converted mistakenly.
  
  if (!tag) return tag;
  
  if (tag.match(/^rdf:\#\$/)) return tag;
  
  var hasColon = tag.match(/(.*?):(.*)/);
  if (hasColon)
  {
    var prefix = hasColon[1];
    var suffix = hasColon[2];
    var nameSpace = this.getNameSpace(prefix);
    
    // prepend the name space in place of the prefix
    if (nameSpace) 
      return nameSpace + suffix;
    
    // it has a colon but the prefix is not registered, 
    // so it is probably already a URI (f.e. http:)
    else
      return tag;
  } 
  // it is in the default namespace
  else 
    return this.defaultNameSpace + tag; 
}

RDFDataSource.prototype.uriToResource = function(uri)
{
  var canonicalURI = this.tagToURI(uri);

  var resource = gRDFService.GetResource(canonicalURI);

  return resource;
}

//----------------------------------------------------------------------------//
// Query Data Source Methods

RDFDataSource.prototype.getSource = function(aPropertyURI, aTargetValue)
{
  // Treat the target as a resource and try to get a source
  // with the given property having the given target value.
  var property = this.uriToResource(aPropertyURI);
  var target = this.uriToResource(aTargetValue);
  var source = this.ds.GetSource(property, target, true);
  
  // If we didn't find a source, try again, this time treating
  // the target as a literal value.
  if (typeof source == "undefined")
  {
    target = gRDFService.GetLiteral(aTargetValue);
    source = this.ds.GetSource(property, target, true);
  }
  
  var sourceValue;
  if (typeof source != "undefined")
    sourceValue = RDFDataSource.nodeValue(source);
  
  return sourceValue;
}

RDFDataSource.prototype.getSources = function(aPropertyURI, aTargetValue)
{
  // Treat the target as a resource and try to get sources
  // with the given property having the given target value.
  var property = this.uriToResource(aPropertyURI);
  var target = this.uriToResource(aTargetValue);
  var enumerator = this.ds.GetSources(property, target, true);

  // If we didn't find sources, try again, this time treating
  // the target as a literal value.
  if (!enumerator.hasMoreElements())
  {
    target = gRDFService.GetLiteral(aTargetValue);
    enumerator = this.ds.GetSources(property, target, true);
  }

  var sources = new Array();
  var source, sourceValue;
  while (enumerator.hasMoreElements())
  {
    source = enumerator.getNext();
    sourceValue = RDFDataSource.nodeValue(source);
    sources.push(sourceValue);
  }

  return sources;
}

RDFDataSource.prototype.getTarget = function(aSourceURI, aPropertyURI)
{
  var source = this.uriToResource(aSourceURI);
  var property = this.uriToResource(aPropertyURI);
  var target = this.ds.GetTarget(source, property, true);

  var targetValue;
  if (typeof target != "undefined")
    targetValue = RDFDataSource.nodeValue(target);

  return targetValue;
}

RDFDataSource.prototype.getTargets = function(aSourceURI, aPropertyURI)
{
  var source = this.uriToResource(aSourceURI);
  var property = this.uriToResource(aPropertyURI);
  var enumerator = this.ds.GetTargets(source, property, true);
  
  var targets = new Array();
  var target, targetValue;
  while (enumerator.hasMoreElements())
  {
    target = enumerator.getNext();
    targetValue = RDFDataSource.nodeValue(target);
    targets.push(targetValue);
  }
  
  return targets;
}

RDFDataSource.prototype.getContainer = function(aSourceURI, aPropertyURI)
{
  var source = this.uriToResource(aSourceURI);
  var property = this.uriToResource(aPropertyURI);
  var container = this.ds.GetTarget(source, property, true);
  
  var wrappedContainer;
  if (typeof container != "undefined" && container)
  {
    container = container.QueryInterface(Components.interfaces.nsIRDFResource);
    if (gRDFContainerService.IsContainer(this.ds, container))
      wrappedContainer = new RDFContainer(this, container);
  }
  
  return wrappedContainer;
}

RDFDataSource.prototype.hasAssertion = function(aSourceURI, aPropertyURI, aTargetValue)
{
  var source = this.uriToResource(aSourceURI);
  var property = this.uriToResource(aPropertyURI);
  var target = this.uriToResource(aTargetValue);

  var hasAssertion = this.ds.HasAssertion(source, property, target, true);
  
  if (!hasAssertion)
  {
    target = gRDFService.ds.GetLiteral(aTargetValue);
    hasAssertion = this.ds.HasAssertion(source, property, target, true);
  }
  
  return hasAssertion;
}

//----------------------------------------------------------------------------//
// Alter Data Source Methods

RDFDataSource.prototype.changeResource = function(aSourceURI, aPropertyURI, aOldTargetURI, aNewTargetURI)
{
  var source = this.uriToResource(aSourceURI);
  var property = this.uriToResource(aPropertyURI);
  var oldTarget = this.uriToResource(aOldTargetURI);
  var newTarget = this.uriToResource(aNewTargetURI);

  this.ds.Change(source, property, oldTarget, newTarget);

  // Return whether or not the assertion was successfully changed.
  return this.ds.HasAssertion(source, property, newTarget, true);
}

RDFDataSource.prototype.changeLiteral = function(aSourceURI, aPropertyURI, aOldTargetLiteral, aNewTargetLiteral)
{
  var source = this.uriToResource(aSourceURI);
  var property = this.uriToResource(aPropertyURI);
  var oldTarget = gRDFService.GetLiteral(aOldTargetLiteral);
  var newTarget = gRDFService.GetLiteral(aNewTargetLiteral);

  this.ds.Change(source, property, oldTarget, newTarget);

  // Return whether or not the assertion was successfully changed.
  return this.ds.HasAssertion(source, property, newTarget, true);
}

RDFDataSource.prototype.assertResource = function(aSourceURI, aPropertyURI, aTargetURI)
{
  var source = this.uriToResource(aSourceURI);
  var property = this.uriToResource(aPropertyURI);
  var target = this.uriToResource(aTargetURI);
  
  this.ds.Assert(source, property, target, true);

  // Return whether or not the assertion was successful.
  return this.ds.HasAssertion(source, property, target, true);
}

RDFDataSource.prototype.assertLiteral = function(aSourceURI, aPropertyURI, aTargetLiteral)
{
  var source = this.uriToResource(aSourceURI);
  var property = this.uriToResource(aPropertyURI);
  var target = gRDFService.GetLiteral(aTargetLiteral);
  
  this.ds.Assert(source, property, target, true);

  // Return whether or not the assertion was successful.
  return this.ds.HasAssertion(source, property, target, true);
}

RDFDataSource.prototype.unassertResource = function(aSourceURI, aPropertyURI, aTargetURI)
{
  var source = this.uriToResource(aSourceURI);
  var property = this.uriToResource(aPropertyURI);
  var target = this.uriToResource(aTargetURI);
  
  this.ds.Unassert(source, property, target, true);

  // Return whether or not the assertion was successful.
  return !this.ds.HasAssertion(source, property, target, true);
}

RDFDataSource.prototype.unassertLiteral = function(aSourceURI, aPropertyURI, aTargetLiteral)
{
  var source = this.uriToResource(aSourceURI);
  var property = this.uriToResource(aPropertyURI);
  var target = gRDFService.GetLiteral(aTargetLiteral);
  
  this.ds.Unassert(source, property, target, true);

  // Return whether or not the assertion was successful.
  return !this.ds.HasAssertion(source, property, target, true);
}

RDFDataSource.prototype.unassertAll = function(aSourceURI)
{
  // Set up a cache to record the URIs of resources we unassert
  // so we don't get stuck in an infinite loop if the resource chain
  // contains a circular reference.
  var deleteCache = false;
  if (typeof this.unassertCache == "undefined")
  {
    this.unassertCache = {};
    deleteCache = true;
  }
  
  // Add this resource to the cache.
  this.unassertCache[aSourceURI] = true;
  
  // Get this resource's properties.
  var source = this.uriToResource(aSourceURI);
  var properties = this.ds.ArcLabelsOut(source);
  
  var property, target, targetType, targetURI;
  while (properties.hasMoreElements())
  {
    property = properties.getNext();
    target = this.ds.GetTarget(source, property, true);
    
    targetType = RDFDataSource.nodeType(target);
    if (targetType == "resource")
    {
      targetURI = RDFDataSource.nodeValue(target);
      if (typeof this.unassertCache[targetURI] == "undefined")
        this.unassertAll(targetURI);
    }
    
    this.ds.Unassert(source, property, target, true);
  }

  if (deleteCache) delete this.unassertCache;
}

RDFDataSource.prototype.makeContainer = function(aSourceURI, aPropertyURI, aContainerType)
{
  // Generate a URI for this container and assert it into the data source
  // as a normal resource.
  var containerURI = this.generateUniqueURI();
  this.assertResource(aSourceURI, aPropertyURI, containerURI);
  
  // Use the container service to turn the resource into a container.
  var container = gRDFService.GetResource(containerURI);
  if      (aContainerType == "Seq")
    gRDFContainerService.MakeSeq(this.ds, container);
  else if (aContainerType == "Bag")
    gRDFContainerService.MakeBag(this.ds, container);
  else if (aContainerType == "Alt")
    gRDFContainerService.MakeAlt(this.ds, container);
  else throw("Unrecognized container type: " + aContainerType);
  
  var wrappedContainer = new RDFContainer(this, container);
  
  // Wrap the container in an RDFContainer wrapper and return it to the caller.
  return wrappedContainer;
}

RDFDataSource.prototype.makeSeq = function(aSourceURI, aPropertyURI)
{
  return this.makeContainer(aSourceURI, aPropertyURI, "Seq");
}

RDFDataSource.prototype.makeBag = function(aSourceURI, aPropertyURI)
{
  return this.makeContainer(aSourceURI, aPropertyURI, "Bag");
}

RDFDataSource.prototype.makeAlt = function(aSourceURI, aPropertyURI)
{
  return this.makeContainer(aSourceURI, aPropertyURI, "Alt");
}


RDFDataSource.prototype.generateUniqueURI = function()
{
  // Generates a random, unique URI for a resource in the format
  // Mozilla uses for container URIs.
  var uri = "rdf:#$" + RDFDataSource.generateRandomString();
  var resource = gRDFService.GetResource(uri);
  var arcsIn = this.ds.ArcLabelsIn(resource);
  var arcsOut = this.ds.ArcLabelsOut(resource);
  while (arcsIn.hasMoreElements() || arcsOut.hasMoreElements())
  {
    uri = "rdf:#$" + RDFDataSource.generateRandomString();
    resource = gRDFService.GetResource(uri);
    arcsIn = this.ds.ArcLabelsIn(resource);
    arcsOut = this.ds.ArcLabelsOut(resource);
  }
  return uri;
}

//----------------------------------------------------------------------------//

function RDFContainer(aDataSource, anXPCOMResource)
{
  if ( !aDataSource )
    throw("RDFContainer: '" + aDataSource + "' is not a data source.");
  if ( !anXPCOMResource || 
         !anXPCOMResource.Value || 
           !gRDFContainerService.IsContainer(aDataSource.ds, anXPCOMResource) )
    throw("RDFContainer: '" + anXPCOMResource + "' is not a container.");
  
  this.uri = anXPCOMResource.Value;
  
  // If the container is already defined, return the existing one;
  // otherwise add it to the list of all containers.
  if ( typeof RDFContainer.all[this.uri] != "undefined" )
  {
    var existingContainer = RDFContainer.all[this.uri];
    existingContainer.reset();
    return existingContainer;
  }
  else 
    RDFContainer.all[this.uri] = this;

  this.dataSource = aDataSource;

  this.XPCOMContainer = 
    Components
      .classes["@mozilla.org/rdf/container;1"]
        .createInstance(Components.interfaces.nsIRDFContainer);
  if (!this.XPCOMContainer) 
    throw("RDFContainer: could not create instance of nsIRDFContainer.");

  this.XPCOMContainer.Init(this.dataSource.ds, anXPCOMResource);

  this.reset();

  return this;
}

// All containers indexed by URI.
RDFContainer.all = new Object();

RDFContainer.prototype.purge = function()
{
  // Removes a container from a data source by removing all its elements
  // and all references to/from it.
  // !!! Make this a generic method of RDFDataSource!
    
  // Remove all items from this container.
  this.removeAll();
  
  var container = gRDFService.GetResource(this.uri);
  
  // Remove all references to this container.
  properties = this.dataSource.ds.ArcLabelsIn(container);
  var sourceURI;
  while (properties.hasMoreElements())
  {
    property = properties.getNext();
    propertyURI = RDFDataSource.nodeValue(property);
    sourceURI = this.dataSource.getSource(propertyURI, this.uri);
    this.dataSource.unassertResource(sourceURI, propertyURI, this.uri);
  }
  
  // Remove all references this container holds.
  var properties = this.dataSource.ds.ArcLabelsOut(container);
  var property, propertyURI, targetValue;
  while (properties.hasMoreElements())
  {
    property = properties.getNext();
    propertyURI = RDFDataSource.nodeValue(property);
    targetValue = this.dataSource.getTarget(this.uri, propertyURI);
    if (targetValue)
    {
      this.dataSource.unassertResource(this.uri, propertyURI, targetValue);
      this.dataSource.unassertLiteral(this.uri, propertyURI, targetValue);
    }
  }
}

//----------------------------------------------------------------------------//

RDFContainer.prototype.reset = function()
{
  this.XPCOMElementsIterator = this.XPCOMContainer.GetElements();
  if ( !this.XPCOMElementsIterator )
    throw("RDFContainer.reset: could not get the elements from the container.");
  return true;
}

RDFContainer.prototype.hasMoreElements = function()
{
  return this.XPCOMElementsIterator.hasMoreElements();
}

RDFContainer.prototype.getNextElement = function(raw)
{
  var element = this.XPCOMElementsIterator.getNext();
  var val = RDFDataSource.nodeValue(element);
  return val;
}

//----------------------------------------------------------------------------//

RDFContainer.prototype.append = function(anElementValue, elementType)
{

  var anElement;
  if (elementType == 'resource') 
    anElement = this.dataSource.uriToResource(anElementValue);
  else if (elementType == 'literal') 
    anElement = gRDFService.GetLiteral(anElementValue);
  else 
    throw("RDFContainer.append: the element type '" + elementType + 
          "' must be either 'resource' or 'literal'.");

  this.XPCOMContainer.AppendElement(anElement);

  // Return whether or not the element was successfully appended.
  var elementIndex = this.XPCOMContainer.IndexOf(anElement);
  return (elementIndex != -1);
}

RDFContainer.prototype.appendResource = function(anElementValue)
{
  return this.append(anElementValue, 'resource');
}

RDFContainer.prototype.appendLiteral = function(anElementValue)
{
  return this.append(anElementValue, 'literal');
}

//----------------------------------------------------------------------------//

RDFContainer.prototype.remove = function(anElementValue, elementType)
{
  var anElement;
  if (elementType == 'resource') 
    anElement = this.dataSource.uriToResource(anElementValue);
  else if (elementType == 'literal') 
    anElement = gRDFService.GetLiteral(anElementValue);
  else 
    throw("RDFContainer.append: the element type '" + elementType + 
          "' must be either 'resource' or 'literal'.");

  this.XPCOMContainer.RemoveElement(anElement, true);

  // Return whether or not the element was successfully removed.
  var elementIndex = this.XPCOMContainer.IndexOf(anElement);
  return (elementIndex == -1);
}

RDFContainer.prototype.removeResource = function(anElementValue)
{
  return this.remove(anElementValue, 'resource');
}

RDFContainer.prototype.removeLiteral = function(anElementValue)
{
  return this.remove(anElementValue, 'literal');
}

RDFContainer.prototype.removeAll = function()
{
  var iterator = this.XPCOMContainer.GetElements();
  var elements = new Array();
  
  // We can't remove elements while looping over them because the iterator
  // does not support changing the container while looping over it, so
  // strange things like not all elements being removed would happen.
  // Instead we get all the elements first and then remove them.
  
  while (iterator.hasMoreElements())
    elements.push(iterator.getNext());
  
  var len = elements.length;
  
  for ( var i=0 ; i<len ; i++ )
    this.XPCOMContainer.RemoveElement(elements[i], true);
}

//----------------------------------------------------------------------------//

RDFContainer.prototype.indexOf = function(anElementValue, elementType)
{
  var anElement;
  if (elementType == 'resource') 
    anElement = this.dataSource.uriToResource(anElementValue);
  else if (elementType == 'literal') 
    anElement = gRDFService.GetLiteral(anElementValue);
  else 
    throw("RDFContainer.append: the element type '" + elementType + 
          "' must be either 'resource' or 'literal'.");

  return this.XPCOMContainer.IndexOf(anElement);
}

RDFContainer.prototype.indexOfResource = function(anElementValue)
{
  return this.indexOf(anElementValue, 'resource');
}

RDFContainer.prototype.indexOfLiteral = function(anElementValue)
{
  return this.indexOf(anElementValue, 'literal');
}

//----------------------------------------------------------------------------//
